#include "HelloWorldScene.h"

using namespace cocos2d;

CCScene* HelloWorld::scene()
{
    CCScene *scene = CCScene::create();
    HelloWorld *layer = HelloWorld::create();
    scene->addChild(layer);
    
    return scene;
}

bool HelloWorld::init()
{
    if (!CCLayer::init())
        return false;
    
    CCSize visibleSize = CCDirector::sharedDirector()->getVisibleSize();
    
    CCSprite* pSprite = CCSprite::create("HelloWorld.png");
    pSprite->setPosition(ccp(visibleSize.width / 2, visibleSize.height / 2));
    this->addChild(pSprite, 0);
    
    srand((unsigned)time(NULL));
    
    usingSprite100();
    //usingSpriteFrameCache();
    //usingTextureCache1();
    //usingTextureCache2();
    
    //usingSprite1000();
    //usingSpriteBatchNode();
    
    return true;
}

void HelloWorld::usingSprite100()
{
    CCSize winSize = CCDirector::sharedDirector()->getWinSize();
    
    start = clock();
    
    for (int i = 0; i < 100; i++) {
        int random = rand();
        
        // ふつうの表示
        CCString* fileName = CCString::createWithFormat("cat%02d.png", i);
        CCSprite* cat = CCSprite::create(fileName->getCString());
        cat->setPosition(ccp(random % (int)winSize.width, random % (int)winSize.height));
        this->addChild(cat);
    }
    
    CCLog("%f", (double)(clock() - start) / CLOCKS_PER_SEC);
}

void HelloWorld::usingSpriteFrameCache()
{
    CCSize winSize = CCDirector::sharedDirector()->getWinSize();
    
    start = clock();
    
    // キャッシュへ読込み
    CCSpriteFrameCache* cache = CCSpriteFrameCache::sharedSpriteFrameCache();
    cache->addSpriteFramesWithFile("cats1.plist");
    cache->addSpriteFramesWithFile("cats2.plist");
    
    for (int i = 0; i < 100; i++) {
        int random = rand();
        
        // キャッシュから表示
        CCString* fileName = CCString::createWithFormat("cat%02d.png", i);
        CCSprite* cat = CCSprite::createWithSpriteFrameName(fileName->getCString());
        cat->setPosition(ccp(random % (int)winSize.width, random % (int)winSize.height));
        this->addChild(cat);
    }
    
    CCLog("%f", (double)(clock() - start) / CLOCKS_PER_SEC);
}

void HelloWorld::usingTextureCache1()
{
    start = clock();
    
    for (int i = 0; i < 100; i++)
    {
        CCString* fileName = CCString::createWithFormat("cat%02d.png", i);
        CCTextureCache* cache = CCTextureCache::sharedTextureCache();
        
        // キャッシュへ非同期で読込み
        cache->addImageAsync(fileName->getCString(), this, callfuncO_selector(HelloWorld::loadedImage));
    }
    
    CCLog("%f", (double)(clock() - start) / CLOCKS_PER_SEC);
}

void HelloWorld::loadedImage(cocos2d::CCObject* object)
{
    CCSize winSize = CCDirector::sharedDirector()->getWinSize();
    
    // キャッシュから表示
    CCTexture2D* texture = (CCTexture2D*)object;
    int random = rand();
    CCSprite* cat = CCSprite::createWithTexture(texture);
    cat->setPosition(ccp(random % (int)winSize.width, random % (int)winSize.height));
    this->addChild(cat);
    
    static int i = 0;
    
    if (++i == 100)
    {
        CCLog("%f", (double)(clock() - start) / CLOCKS_PER_SEC);
    }
}

void HelloWorld::usingTextureCache2()
{
    start = clock();
    
    CCTextureCache* cache = CCTextureCache::sharedTextureCache();
    
    // キャッシュへ非同期で読込み
    cache->addImageAsync("cats1.png", this, callfuncO_selector(HelloWorld::loadedImageCats1));
    cache->addImageAsync("cats2.png", this, callfuncO_selector(HelloWorld::loadedImageCats2));
    
    CCLog("%f", (double)(clock() - start) / CLOCKS_PER_SEC);
}

void HelloWorld::loadedImageCats1(cocos2d::CCObject* object)
{
    CCSpriteFrameCache* cache = CCSpriteFrameCache::sharedSpriteFrameCache();
    cache->addSpriteFramesWithFile("cats1.plist", (CCTexture2D*)object);
    
    loadedImageCats(1);
}

void HelloWorld::loadedImageCats2(cocos2d::CCObject* object)
{
    CCSpriteFrameCache* cache = CCSpriteFrameCache::sharedSpriteFrameCache();
    cache->addSpriteFramesWithFile("cats2.plist", (CCTexture2D*)object);
    
    loadedImageCats(2);
}

void HelloWorld::loadedImageCats(int no)
{
    CCSize winSize = CCDirector::sharedDirector()->getWinSize();
    int j = (no == 1) ? 0 : 50;
    for (int i = 0; i < 50; i++)
    {
        int random = rand();
        
        // キャッシュから表示
        CCString* fileName = CCString::createWithFormat("cat%02d.png", i + j);
        CCSprite* cat = CCSprite::createWithSpriteFrameName(fileName->getCString());
        cat->setPosition(ccp(random % (int)winSize.width, random % (int)winSize.height));
        this->addChild(cat);
    }
    
    CCLog("%f", (double)(clock() - start) / CLOCKS_PER_SEC);
}

void HelloWorld::usingSprite1000()
{
    CCSize winSize = CCDirector::sharedDirector()->getWinSize();
    
    start = clock();
    
    for (int i = 0; i < 1000; i++) {
        int random = rand();
        
        // ふつうの表示
        CCString* fileName = CCString::createWithFormat("cat00.png", i);
        CCSprite* cat = CCSprite::create(fileName->getCString());
        cat->setPosition(ccp(random % (int)winSize.width, random % (int)winSize.height));
        this->addChild(cat);
    }
    
    CCLog("%f", (double)(clock() - start) / CLOCKS_PER_SEC);
}

void HelloWorld::usingSpriteBatchNode()
{
    CCSize winSize = CCDirector::sharedDirector()->getWinSize();
    
    start = clock();
    
    // バッチノード登録
    CCSpriteBatchNode* batchNode = CCSpriteBatchNode::create("cat00.png");
    this->addChild(batchNode);
    
    for (int i = 0; i < 1000; i++) {
        int random = rand();
        
        // バッチノードより画像を読込み表示
        CCSprite *cat = CCSprite::createWithTexture(batchNode->getTexture());
        cat->setPosition(ccp(random % (int)winSize.width, random % (int)winSize.height));
        batchNode->addChild(cat);
    }
    
    CCLog("%f", (double)(clock() - start) / CLOCKS_PER_SEC);
}
